'use strict';

const Q = require('q');
const co = require('../co');
const fs = require('fs');
const path = require('path');
const http = require('http');
const gutil = require('gulp-util');
const querystring = require('querystring');
const yosay = require('yosay');    // yeoman弹出框
const utils = require('../utils.js');
const config = JSON.parse(fs.readFileSync('build/config.json','utf-8'));
const cmsHost = 'cms.4399houtai.com';
const flowTask = require('./flow');

module.exports = {


	createChannel: co.wrap(function *() {



		// 生成cms栏目
		const makeChannel = co.wrap(function *(config) {

			if (!config.cmsChannel || !config.cmsTemplate) {
				gutil.log(yosay('请配置cms栏目信息\r\n【config.json -> cmsChannel】 \r\n【config.json -> cmsTemplate】'));
				return false;
			}

			var data = {
				name: config.name,
				classify_src: config.cmsChannel,
				list_template: config.cmsTemplate,
				content_template: config.cmsTemplate,
				order_index: '',
				state: '0',
				default_file: 'index.html',
				extend: ''
			};

			var params = querystring.stringify({
				game: config.game,
				sub: config.cmsParentChannel,
				data: JSON.stringify(data)

			});

			var options = {
				hostname: cmsHost,
				path: '/?ct=classify&ac=save',
				method: 'POST',
				headers: {
					'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
					'Content-Length': Buffer.byteLength(params)
				}
			}

			let chunk = yield utils.httpRequest(options, params);

			if (chunk.cid) {
				fs.writeFileSync('build/api.json', JSON.stringify(chunk, null, 4));
				gutil.log("api.json 写入成功！");
			}
			else {
				gutil.log(yosay('出错了！ ' + chunk.tips));
			}

			return Promise.resolve();
		});


		let name = yield flowTask.makeConfig();
		if (!name) {
			gutil.log(yosay('请配置该活动的名称\r\n【config.json -> name】'));
			return false;
		}

		// 重新获取一次config.json
		let config = JSON.parse(fs.readFileSync('build/config.json','utf-8'));

		yield makeChannel(config);

		return Promise.resolve();


	}),


	initTpl: co.wrap(function *() {


		var defered = Q.defer();

		var api = JSON.parse(fs.readFileSync('build/api.json','utf-8'));

		if (!api.channel) {
			gutil.log(yosay('请先执行init方法获得cms栏目id'));
			defered.reject();
			return ;
		}

		var params = querystring.stringify({
			cid: api.channel,
			file: config.cmsTemplate,
			code: 'just for init'

		});

		var options = {
			hostname: cmsHost,
			path: '/?ct=tpl&ac=save',
			method: 'POST',
			headers: {
				'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
				'Content-Length': Buffer.byteLength(params)
			}
		}

		let chunk = yield utils.httpRequest(options, params);

		if (chunk == 'ok') {
			defered.resolve();
		}
		else {
			gutil.log(yosay('保存模板失败：' + chunk));
			defered.reject();
		}

		return defered.promise;
	}),

	saveTpl: co.wrap(function *() {


		var defered = Q.defer();

		var api = JSON.parse(fs.readFileSync('build/api.json','utf-8'));
		var fileData = fs.readFileSync('dist/index.html','utf-8');

		if (!api.channel) {
			gutil.log(yosay('请先执行init方法获得cms栏目id'));
			defered.reject();
			return ;
		}

		var params = querystring.stringify({
			cid: api.channel,
			file: config.cmsTemplate,
			code: fileData

		});

		var options = {
			hostname: cmsHost,
			path: '/?ct=tpl&ac=save',
			method: 'POST',
			headers: {
				'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
				'Content-Length': Buffer.byteLength(params)
			}
		}

		let chunk = yield utils.httpRequest(options, params);

		if (chunk == 'ok') {
			defered.resolve();
		}
		else {
			gutil.log(yosay('保存模板失败：' + chunk));
			defered.reject();
		}

		return defered.promise;
	}),

	makeTpl: co.wrap(function *() {

		var deferred = Q.defer();


		if (!config.cid) {
			gutil.log('config.json缺少cid。');

			config.cid = yield utils.readSyncByRl('请录入活动后台cid（后续可在build/config.json中进行手动修改）：');

			if (!config.cid) {
				deferred.reject();
				return ;
			}
			else {
				config.cid = `20${config.cid}`;
				fs.writeFileSync('build/config.json', JSON.stringify(config, null, 4));
				gutil.log('cid录入成功。正在生成模板...');
			}
		}

		// 生成模板
		const makeTpl = co.wrap(function *() {
			let deferred = Q.defer();

			let config = JSON.parse(fs.readFileSync('build/api.json','utf-8'));
			let params = querystring.stringify({
				channel: config.channel,
				cid: config.cid,
				just_list: 0,
				post_date: 1

			});

			let options = {
				hostname: cmsHost,
				path: '/?ct=view&ac=taskpro',
				method: 'POST',
				headers: {
					'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
					'Content-Length': Buffer.byteLength(params)
				}
			};

			let chunk = yield utils.httpRequest(options, params);

			if (chunk.state == true) {
				var sign = chunk.data;
				deferred.resolve(sign);
			}
			else {
				gutil.log(yosay('模板生成失败: '));
			}

			return deferred.promise;

		});

		// 模板生成进度
		const makePro = co.wrap(function *(sign) {

			let deferred = Q.defer();

			let taskProcess = setInterval(co.wrap(function *() {

				let params = querystring.stringify({
					sign: sign
				});

				let options = {
					hostname: cmsHost,
					path: '/?ct=view&ac=taskFinish&' + params,
					method: 'GET'
				};

				let chunk = yield utils.httpRequest(options, params);

				gutil.log(Math.ceil(chunk.data * 100) + '%');

				if (chunk.data == 1) {
					clearInterval(taskProcess);
					deferred.resolve();
				}

			}), 1000);

			return deferred.promise;

		});

		// 模板生成是否成功
		const makeError = co.wrap(function *(sign) {

			let deferred = Q.defer();

			let params = querystring.stringify({
				sign: sign
			});

			let options = {
				hostname: cmsHost,
				path: '/?ct=view&ac=taskError&' + params,
				method: 'GET'
			};

			let chunk = yield utils.httpRequest(options, params);

			if (chunk.data.length == 0) {
				gutil.log('模板生成成功。');
				deferred.resolve(1);
			}
			else {
				gutil.log('模板生成失败: ' + chunk.data[0].msg + ' , 详情请点击 ' + chunk.data[0].url);
				deferred.resolve(0);
			}


			return deferred.promise;
		});

		let sign = yield makeTpl();
		yield makePro(sign);
		let flag = yield makeError(sign);

		// flag == 1 生成成功，否则失败
		if (flag == 1) {
			deferred.resolve();
		}

		return deferred.promise;
	}),

	// 推送模板到线上服务器
	pushTpl: co.wrap(function *() {

		var deferred = Q.defer();
		let config = JSON.parse(fs.readFileSync('build/api.json','utf-8'));
		let params = querystring.stringify({
			cid: config.channel
		});

		let options = {
			hostname: cmsHost,
			path: '/?ct=view&ac=rsync&' + params,
			method: 'GET'
		};

		let chunk = yield utils.httpRequest(options, params);

		if (chunk.state == false) {
			gutil.log(yosay('推送失败: ' + chunk.msg));
		}
		else {
			gutil.log('模板推送成功，' + chunk.siteurl);
		}

		deferred.resolve();

		return deferred.promise;

	})


}