'use strict';
const co = require('../co');
const path = require('path');
const fs = require('fs');
const gulp = require('gulp');
const gutil = require('gulp-util');
const browserSync = require('browser-sync').create();
const opn = require('opn');
const rev = require('gulp-rev');
const revCollector = require('gulp-rev-collector');
const replace = require('gulp-replace');
const uglify = require('gulp-uglify');
const transport = require('gulp-seajs-transport');
const seaConcat = require('gulp-seajs-concat');
const concat = require('gulp-concat');
const rename = require('gulp-rename');
const mergeStream = require("merge-stream");
// var imagemin = require('imagemin');
// var imageminMozjpeg = require('imagemin-mozjpeg');
// var imageminPngquant = require('imagemin-pngquant');

const imagemin = require('gulp-imagemin');
const smushit = require('gulp-smushit');
const pngquant = require('imagemin-pngquant'); // 深度压缩

const config = JSON.parse(fs.readFileSync('build/config.json','utf-8'));
const devConfig = JSON.parse(fs.readFileSync('build/dev.json','utf-8'));
const utils = require('../utils.js');

// 以下是路径相关变量
let dirArrs = process.cwd().split(path.sep);
let cmsChannel = (new Date()).getFullYear() + dirArrs[dirArrs.length - 1];
let cmsIndex = dirArrs.indexOf('cms');
let game = dirArrs[cmsIndex + 1];

let cmsParentChannel = config.cmsParentChannel;
if (game == 'hd') {
	game = 'web';
}


module.exports = {

	// 初始化config.json
	makeConfig: co.wrap(function *() {

		let name = config.name;
		let cmsParentChannel = config.cmsParentChannel;
		if (!config.name) {
			name = yield utils.readSyncByRl('请输入活动名称（后续可在build/config.json中进行手动修改）：');
		}

		if (!config.cmsParentChannel) {
			cmsParentChannel = yield utils.readSyncByRl('请输入cms父栏目，不录入默认为空，即根栏目（后续可在build/config.json中进行手动修改）：');
		}

		return new Promise((resolve) => {

			let configJson = {
				cid: 0,
				name: name,
				game: game,
				cmsParentChannel: cmsParentChannel,
				cmsChannel: cmsChannel,
				cmsTemplate: cmsChannel + '.html',
				entry: 'entry',
				tsIgnore: [
					'config*',
					'jq*'
				]
			}
			fs.writeFileSync('build/config.json', JSON.stringify(configJson, null, 4));


			resolve(name);


		})


	}),

	// seajs匿名模块转为具名模块
	jsTransport() {

		let srcArr = ['js/*.js'];
		for (let item of config.tsIgnore) {
			srcArr.push('!js/' + item);
		}

		srcArr.push('!js/entry');

		gutil.log(config.entry + '.js 更新成功');

		var stream1 = gulp.src(srcArr)
			.pipe(transport())
		// .pipe(concat('entry.js'));

		var stream2 = gulp.src(`js/entry.js`)
		// .pipe(gulp.dest('dist'))
		var stream3 = mergeStream(stream1, stream2);
		return stream3.pipe(concat(config.entry + '.js'))
			.pipe(gulp.dest('dist/js'))
			.pipe(concat(config.entry.split('-')[0] + '.js'))
			.pipe(gulp.dest('dist/js'));
		// return gulp.src(srcArr, {base: 'js'})
		// 	.pipe(transport())
		// 	.pipe(concat(config.entry + '.js'))
		// 	.pipe(gulp.dest('dist/js'))
		// 	.pipe(concat(config.entry.split('-')[0] + '.js'))
		// 	.pipe(gulp.dest('dist/js'));

	},

	// 初始化路径信息
	initPath() {

		let config = JSON.parse(fs.readFileSync('build/config.json','utf-8'));

		let cmsChannel = config.cmsChannel;
		let cmsParentChannel = config.cmsParentChannel;
		let cdnPath = '';					// 正式环境cdn路径
		let cdnDevPath = '';				// 测试环境cdn路径
		let localPathUri = '';         		// 【本地】模板路径uri
		let proPathUrl = '';				// 【测试或正式环境】模板路径uri

		// 通过路径的规则，判断所属游戏，以及构建cdn路径以及模板路径

		localPathUri = dirArrs.reduce((item, item2, index) => {
			if (index <= cmsIndex) {
				return '';
			}
			else {
				return `${item}/${item2}`;
			}
		});

		cdnPath = 'http://pic.my4399.com/re/cms' + localPathUri + '/';
		cdnDevPath = 'http://pic.dev.4399.com/re/cms' + localPathUri + '/';


		if (game == 'web') {
			proPathUrl = 'hd/' + cmsChannel + '/';
		}
		else if (cmsParentChannel == '') {
			proPathUrl = game + '/' + cmsChannel + '/';
		}
		else {
			proPathUrl = `${game}/${cmsParentChannel}/${cmsChannel}/`;
		}

		let devConfig = {
			cdnPath: cdnPath,
			cdnDevPath: cdnDevPath,
			localUrl: 'http://web.local.4399.com' + localPathUri,
			devUrl: 'http://web.dev.4399.com/' + proPathUrl,
			proUrl: 'http://web.4399.com/' + proPathUrl,
		};

		fs.writeFile('build/dev.json', JSON.stringify(devConfig, null, 4), function(err) {
			if (!err) {
				// gutil.log("dev.json 写入成功！")
			}
		});


		let fileList = utils.walk('js');
		if (fileList.indexOf('entry.js') === -1) {
			gulp.src('build/entry.js')
				.pipe(gulp.dest('js'));
		}

	},

	// 打开本地环境
	browerLocal() {
		let _this = this;
		browserSync.init({
			port: 3002
		});

		opn(devConfig.localUrl);

		gulp.watch('*.html').on('change', browserSync.reload);
		gulp.watch('js/*.js').on('change', function() {
			_this.jsTransport();
			setTimeout(function() {
				// _this.jsMin();
				browserSync.reload();
			}, 1000);

		});
	},

	// 打开测试环境
	browerDev() {
		opn(devConfig.devUrl);
	},

	// 打开正式环境
	browerPro() {
		opn(devConfig.proUrl);
	},

	// 压缩图片
	imageMin() {
		gulp.src('./images/*.jpg')
		// .pipe(smushit({
		// 	verbose: true
		// }))
			.pipe(imagemin({
				progressive: true,
				svgoPlugins: [{removeViewBox: false}],
				use: [pngquant()]
			}))
			.pipe(gulp.dest('./images/'));


	},

	// 压缩js
	jsMin() {
		return gulp.src('dist/js/*.js')
			.pipe(uglify({
				mangle: {except: ['require' ,'exports' ,'module' ,'$']} // 排除混淆关键字
			}))
			.pipe(gulp.dest('dist/js'));
	},

	// 生成版本号json
	jsRev() {

		let revPath = 'rev-manifest-' + utils.getDateString() + utils.getHourString() + '.json';

		return gulp.src('dist/js/' + config.entry.split('-')[0] + '.js')
			.pipe(rev())
			.pipe(gulp.dest('dist/js/'))
			.pipe(rev.manifest({
				path: revPath
			}))
			.pipe(replace('.js', ''))
			.pipe(gulp.dest('build/rev/js'));



	},


	// 覆盖式发布 － 替换版本号
	jsReplace() {

		return gulp.src('dist/js/' + config.entry.split('-')[0] + '.js')
			.pipe(replace(config.entry.split('-')[0], config.entry))
			.pipe(rename(config.entry + '.js'))
			.pipe(gulp.dest('dist/js/'));

	},

	// 非覆盖式发布 － 替换版本号
	jsRevReplace() {

		let revStamp = 'rev-manifest-' + utils.revMax() + '.json';

		let revConfig = JSON.parse(fs.readFileSync('build/rev/js/' + revStamp,'utf-8'));

		config.entry = revConfig[config.entry.split('-')[0]];


		fs.writeFile('build/config.json', JSON.stringify(config, null, 4), function(err) {
			if (!err) {
				gutil.log("config.json 更新版本成功");
			}
		})



		return gulp.src(['build/rev/js/' + revStamp, 'dist/js/' + config.entry + '.js'])
			.pipe(revCollector({
				replaceReved: true,
			}))
			.pipe(gulp.dest('dist/js/'));
	}

	// jsRev() {
	// 	gulp.src(['js/*.js', '!js/config.js'])
	// 		.pipe(replace(/seajs.use([\s\S]*)[.js]{0}[\']{0,1}["]{0,1}\);/, "seajs.use(window.cdnPath + 'dist/js/"))
	// 		.pipe(rev())
	// 		.pipe(gulp.dest('dist/js/'))
	// 		.pipe(rev.manifest({
	// 			path: 'rev-manifest-' + utils.getDateString() + utils.getHourString() + '.json'
	// 		}))
	// 		.pipe(gulp.dest('build/rev/js'));
	//
	// 	let revStamp = 'rev-manifest-' + utils.revMax() + '.json';
	//
	// 	return gulp.src(['build/rev/js/' + revStamp, 'js/config.js'])
	// 		.pipe(revCollector({
	// 			replaceReved: true,
	// 		}) )
	// 		.pipe(replace(/\"/g, "'"))
	// 		.pipe(replace('/main\'', '/main.js\''))
	// 		.pipe(replace('/index\'', '/index.js\''))
	// 		.pipe(replace(/seajs.use([\s\S]*)js\//, "seajs.use(window.cdnPath + 'dist/js/"))
	// 		.pipe(replace('./js/', devConfig.cdnPath + 'dist/js/'))
	// 		.pipe(gulp.dest('dist/js/'));
	// },

	// jsCopy() {
	// 	gulp.src('./js/config.js')
	// 		.pipe(replace(/\"/g, "'"))
	// 		.pipe(replace('/main\'', '/main.js\''))
	// 		.pipe(replace('/index\'', '/index.js\''))
	// 		.pipe(replace(/seajs.use([\s\S]*)js\//, "seajs.use(window.cdnPath + 'dist/js/"))
	// 		.pipe(gulp.dest('./dist/js/'));
	//
	// 	return gulp.src(['./js/*.js', '!./js/config.js'])
	// 		.pipe(gulp.dest('./dist/js/'));
	// }

}
