"use strict";

const gulp = require('gulp');
const replace = require('gulp-replace');
const fs = require('fs');
const rename = require('gulp-rename');
const del = require('del');
const utils = require('../utils.js');
const co = require('../co');

const config = JSON.parse(fs.readFileSync('build/config.json','utf-8'));
const devConfig = JSON.parse(fs.readFileSync('build/dev.json','utf-8'));
const configPath = 'http://pic.my4399.com/re/cms/feUtil/seajs/3.0.0/config.js';

module.exports = {

	initHtml: co.wrap(function *() {


		var options = {
			hostname: 'pic.my4399.com',
			path: '/re/cms/feUtil/seajs/3.0.0/config.js',
			method: 'GET'
		};

		let seaConfig = yield utils.httpRequest(options);



		gulp.src('./*.html')
			.pipe(replace('href="#"', 'href="javascript:;"'))
			.pipe(replace('href=""', 'href="javascript:;"'))

			.pipe(replace(/<script[^>]+>([^<]*)window.cdnPath([^<]*)<\/script>/g, ''))
			.pipe(replace(/<script id="__cdn_script__">([^<]*)<\/script>/g, ''))
			.pipe(replace(/<script id="__sea_script__">([^<]*)<\/script>/g, ''))
			.pipe(replace(/<script id="__bs_script__">(([\s\S]*))<\/script>/g, ''))

			.pipe(replace(/feUtil\/seajs[^>]+>([^<]*)<\/script>/, `feUtil/seajs/3.0.0/sea.js" ><\/script>\n<script id="__cdn_script__">window.cdnPath = \'./\';<\/script>\n<script id="__config_script__" >${seaConfig}<\/script>\n<script id="__sea_script__">seajs.use(window.cdnPath + "dist/js/${config.entry}");</script>`))
			.pipe(replace(/<script([^<]*)config.js"><\/script>/, ''))
			.pipe(replace(/<script([^<]*)config.js" ><\/script>/, ''))

			.pipe(replace(/<link href="http:\/\/webpic.my4399.com\/easydialog.css" rel="stylesheet" type="text\/css" \/>[\n \r\n]{0,}/g, ''))
			.pipe(replace('</title>', '</title>\n 	<link href="http://webpic.my4399.com/easydialog.css" rel="stylesheet" type="text/css" />'))
			.pipe(replace('</body>', '<script id="__bs_script__">var lowIE = !-[1,];if (!lowIE && location.href.indexOf(\'local.4399.com\') !== -1) {document.write(\'<script async src="http://HOST:3002/browser-sync/browser-sync-client.js?v=2.18.8"><\\/script>\'.replace("HOST", location.hostname))};</script>\n</body>'))
			.pipe(gulp.dest('./'));


		return gulp.src('./web.html')
			.pipe(rename('index.html'))
			.pipe(gulp.dest('./'));

	}),

	buildHtml(isDev) {

		let game = config.game;
		let cdnPath = isDev == 1 ? devConfig.cdnDevPath : devConfig.cdnPath;
		let cdnDevPath = devConfig.cdnDevPath;
		let timeStamp = new Date().getTime();

		let cdnScript = '<script id="__cdn_script__">if (location.href.indexOf(\'dev.4399.com\') !== -1) {window.cdnPath = \'' + cdnDevPath + '\';} else {window.cdnPath = \'' + cdnPath + '\';}</script>';


		return gulp.src('./*.html')
			.pipe(replace(/<script id="__cdn_script__">([^<]*)<\/script>/g, cdnScript))
			.pipe(replace('<\/title>', '</title>\n	<script>window.tjCid = ' + config.cid + '; window.tjGame = \'' + game + '\';</script>\n'))
			.pipe(replace(/<script id="__cdnPath"[^>]+>([^<]*)<\/script>/g, ''))
			.pipe(replace('src="js/', 'src="' + cdnPath + 'dist/js/'))
			.pipe(replace('src="./js/', 'src="' + cdnPath + 'dist/js/'))
			.pipe(replace('style/', cdnPath + 'style/'))
			.pipe(replace('href="css/', 'href="' + cdnPath + 'css/'))
			.pipe(replace('./style/', cdnPath + 'style/'))
			.pipe(replace('./css/', cdnPath + 'css/'))
			.pipe(replace('src="images/', 'src="' + cdnPath + 'images/'))
			.pipe(replace('src="./images/', 'src="' + cdnPath + 'images/'))
			.pipe(replace('src="img/', 'src="' + cdnPath + 'img/'))
			.pipe(replace('src="./img/', 'src="' + cdnPath + 'img/'))
			.pipe(replace('src="video/', 'src="' + cdnPath + 'video/'))
			.pipe(replace('src="./video/', 'src="' + cdnPath + 'video/'))
			.pipe(replace('../images/', '' + cdnPath + 'images/'))
			.pipe(replace('src="temp/', 'src="' + cdnPath + 'temp/'))
			.pipe(replace('src="./temp/', 'src="' + cdnPath + 'temp/'))
			.pipe(replace('./', cdnPath))
			.pipe(replace('swf/', cdnPath + 'swf/'))
			.pipe(replace('./swf/', cdnPath + 'swf/'))
			.pipe(replace(/<script id="__sea_script__">([^<]*)<\/script>/, `<script id="__sea_script__">seajs.use(window.cdnPath + "dist/js/${config.entry}.js?v=${timeStamp}");<\/script>`))
			// .pipe(replace(/seajs.use(([\s\S]*));/, `seajs.use("${config.entry}.js?v=${(new Date).getTime()}");`))
			.pipe(replace(/<script id="__bs_script__">((?!<\/body>).)*/, ''))
			// .pipe(inlineImg(''))
			.pipe(gulp.dest('dist'));

	}

}


